@@ -28,6 +28,7 @@
 
 #include "vinagre-vnc-tab.h"
 #include "vinagre-vnc-connection.h"
+#include "vinagre-vnc-tunnel.h"
 
 #define VINAGRE_VNC_TAB_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), VINAGRE_TYPE_VNC_TAB, VinagreVncTabPrivate))
 
@@ -103,6 +104,12 @@ view_original_size_cb (GtkAction *action, VinagreVncTab *vnc_tab)
   vinagre_vnc_tab_original_size (vnc_tab);
 }
 
+static void
+view_refresh_cb (GtkAction *action, VinagreVncTab *vnc_tab)
+{
+  vnc_display_request_update (VNC_DISPLAY (vnc_tab->priv->vnc));
+}
+
 const static GSList *
 vnc_get_connected_actions (VinagreTab *tab)
 {
@@ -236,20 +243,35 @@ vinagre_vnc_tab_class_init (VinagreVncTabClass *klass)
   g_type_class_add_private (object_class, sizeof (VinagreVncTabPrivate));
 }
 
+static gboolean
+idle_close (VinagreTab *tab)
+{
+  vinagre_notebook_close_tab (vinagre_tab_get_notebook (tab), tab);
+  return FALSE;
+}
+
 static void
 open_vnc (VinagreVncTab *vnc_tab)
 {
-  gchar      *host, *port_str;
-  gint       port, shared;
-  gboolean   scaling;
+  gchar      *host, *port_str, *ssh_tunnel_host;
+  gint       port, shared, fd, depth_profile;
+  gboolean   scaling, success, lossy_encoding;
+  GError     *error;
   VncDisplay *vnc = VNC_DISPLAY (vnc_tab->priv->vnc);
   VinagreTab *tab = VINAGRE_TAB (vnc_tab);
 
+  success = TRUE;
+  error = NULL;
+
   g_object_get (vinagre_tab_get_conn (tab),
 		"port", &port,
 		"host", &host,
 		"scaling", &scaling,
 		"shared", &shared,
+		"fd", &fd,
+		"depth-profile", &depth_profile,
+		"lossy-encoding", &lossy_encoding,
+		"ssh-tunnel-host", &ssh_tunnel_host,
 		NULL);
 
   port_str = g_strdup_printf ("%d", port);
@@ -260,16 +282,40 @@ open_vnc (VinagreVncTab *vnc_tab)
 
   vnc_display_set_shared_flag (vnc, shared);
   vnc_display_set_force_size (vnc, !scaling);
+  vnc_display_set_depth (vnc, depth_profile);
+  vnc_display_set_lossy_encoding (vnc, lossy_encoding);
 
-  if (vnc_display_open_host (vnc, host, port_str))
+  if (fd > 0)
+    success = vnc_display_open_fd (vnc, fd);
+  else
+    {
+      if (ssh_tunnel_host && *ssh_tunnel_host)
+	if (!vinagre_vnc_tunnel_create (&host, &port_str, ssh_tunnel_host, &error))
+	  {
+	    success = FALSE;
+	    vinagre_utils_show_error (_("Error creating the SSH tunnel"),
+				      error ? error->message : _("Unknown reason"),
+				      GTK_WINDOW (vinagre_tab_get_window (tab)));
+	    goto out;
+	  }
+      success = vnc_display_open_host (vnc, host, port_str);
+    }
+
+  if (success)
     gtk_widget_grab_focus (GTK_WIDGET (vnc));
   else
-    vinagre_utils_show_error (NULL,
-			      _("Error connecting to host."),
+    vinagre_utils_show_error (_("Error connecting to host."),
+			      error ? error->message : _("Unknown reason"),
 			      GTK_WINDOW (vinagre_tab_get_window (tab)));
 
+out:
   g_free (port_str);
   g_free (host);
+  g_free (ssh_tunnel_host);
+  g_clear_error (&error);
+
+  if (!success)
+    g_idle_add ((GSourceFunc)idle_close, vnc_tab);
 }
 
 static void
@@ -375,12 +421,12 @@ vnc_initialized_cb (VncDisplay *vnc, VinagreVncTab *vnc_tab)
 
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (vnc_tab->priv->scaling_action), scaling);
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (vnc_tab->priv->viewonly_action), view_only);
-  vnc_display_set_pointer_local (VNC_DISPLAY(vnc_tab->priv->vnc), TRUE);
-  vnc_display_set_keyboard_grab (VNC_DISPLAY(vnc_tab->priv->vnc), TRUE);
-  vnc_display_set_pointer_grab (VNC_DISPLAY(vnc_tab->priv->vnc), TRUE);
+  vnc_display_set_pointer_local (vnc, TRUE);
+  vnc_display_set_keyboard_grab (vnc, TRUE);
+  vnc_display_set_pointer_grab (vnc, TRUE);
 
   vinagre_vnc_connection_set_desktop_name (VINAGRE_VNC_CONNECTION (conn),
-					   vnc_display_get_name (VNC_DISPLAY (vnc_tab->priv->vnc)));
+					   vnc_display_get_name (vnc));
 
   name = vinagre_connection_get_best_name (conn);
   label = g_object_get_data (G_OBJECT (tab), "label");
@@ -429,7 +475,7 @@ ask_credential (VinagreVncTab *vnc_tab,
   VinagreTab      *tab = VINAGRE_TAB (vnc_tab);
   VinagreConnection *conn = vinagre_tab_get_conn (tab);
 
-  xml = vinagre_utils_get_builder ();
+  xml = vinagre_utils_get_builder (NULL, NULL);
 
   password_dialog = GTK_WIDGET (gtk_builder_get_object (xml, "auth_required_dialog"));
   gtk_window_set_transient_for (GTK_WINDOW(password_dialog),
@@ -673,6 +719,19 @@ create_connected_actions (VinagreVncTab *tab)
   list = g_slist_append (list, a);
   tab->priv->original_size_action = a->action;
 
+  /* View->Refresh */
+  a = g_new (VinagreTabUiAction, 1);
+  a->paths = g_new (gchar *, 2);
+  a->paths[0] = g_strdup ("/MenuBar/ViewMenu");
+  a->paths[1] = NULL;
+  a->action = gtk_action_new ("VNCViewRefresh",
+			      _("_Refresh Screen"),
+			      _("Requests an update of the screen"),
+			      "gtk-refresh");
+  gtk_action_set_icon_name (a->action, "gtk-refresh");
+  g_signal_connect (a->action, "activate", G_CALLBACK (view_refresh_cb), tab);
+  list = g_slist_append (list, a);
+
   return list;
 }
 
@@ -761,9 +820,8 @@ setup_toolbar (VinagreVncTab *vnc_tab)
   vnc_tab->priv->viewonly_button = button;
 
   /* Send Ctrl-alt-del */
-  button = GTK_WIDGET (gtk_tool_button_new (NULL, NULL));
+  button = GTK_WIDGET (gtk_tool_button_new (NULL, _("Send Ctrl-Alt-Del")));
   gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (button), "preferences-desktop-keyboard-shortcuts");
-  gtk_tool_button_set_label (GTK_TOOL_BUTTON (button), _("Send Ctrl-Alt-Del"));
   gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (button), _("Send Ctrl-Alt-Del"));
   g_signal_connect (button, "clicked", G_CALLBACK (cad_button_clicked), vnc_tab);
   gtk_widget_show (GTK_WIDGET (button));
