@@ -136,23 +136,56 @@ vinagre_utils_get_ui_xml_filename (void)
     return VINAGRE_DATADIR "/" VINAGRE_UI_XML_FILE;
 }
 
+/**
+ * vinagre_utils_get_builder:
+ * @plugin: a #VinagrePlugin or NULL
+ * @filename: the filename of the UI file for the plugin, or NULL if plugin is NULL
+ *
+ * This function gets a #GtkBuilder object for a UI file.
+ * Supply a plugin and a filename in order to get a #GtkBuilder for the plugin.
+ * Supply NULL to both arguments to get the main Vinagre #GtkBuilder
+ *
+ * Returns a #GtkBuilder on success (free with #g_object_unref) or NULL
+ *  if the file cannot be found. In this case an error dialog will be shown.
+ */
 GtkBuilder *
-vinagre_utils_get_builder (void)
+vinagre_utils_get_builder (VinagrePlugin *plugin, const gchar *filename)
 {
-  GtkBuilder *xml;
+  GtkBuilder *xml = NULL;
   GError     *error = NULL;
+  gchar      *actual_filename, *plugin_datadir;
+
+  if (plugin)
+    {
+      plugin_datadir = vinagre_plugin_get_data_dir (plugin);
+      actual_filename = g_build_filename (plugin_datadir, filename, NULL);
+      g_free (plugin_datadir);
+    }
+  else
+    actual_filename = g_strdup (vinagre_utils_get_ui_filename ());
 
   xml = gtk_builder_new ();
   if (!gtk_builder_add_from_file (xml,
-				  vinagre_utils_get_ui_filename (),
+				  actual_filename,
 				  &error))
     {
-      g_error (_("Could not load builder file: %s"), error->message);
+      GString *str = g_string_new (NULL);
+
+      if (plugin)
+	g_string_append (str, _("A plugin tried to open an UI file but did not succeed, with the error message:"));
+      else
+	g_string_append (str, _("The program tried to open an UI file but did not succeed, with the error message:"));
+
+      g_string_append_printf (str, "\n\n%s\n\n", error->message);
+      g_string_append (str, _("Please check your installation."));
+      vinagre_utils_show_error (_("Error loading UI file"), str->str, NULL);
       g_error_free (error);
+      g_string_free (str, TRUE);
       g_object_unref (xml);
-      return NULL;
+      xml = NULL;
     }
 
+  g_free (actual_filename);
   return xml;
 }
 
@@ -514,4 +547,65 @@ vinagre_gtk_button_new_with_stock_icon (const gchar *label,
         return button;
 }
 
+/**
+ * vinagre_utils_ask_question:
+ * @parent: transient parent, or NULL for none
+ * @message: The message to be displayed, if it contains multiple lines,
+ *  the first one is considered as the title.
+ * @choices: NULL-terminated array of button's labels of the dialog
+ * @choice: Place to store the selected button. Zero is the first.
+ *
+ * Displays a dialog with a message and some options to the user.
+ *
+ * Returns TRUE if the user has selected any option, FALSE if the dialog
+ *  was canceled.
+ */
+gboolean
+vinagre_utils_ask_question (GtkWindow  *parent,
+			    const char *message,
+			    char       **choices,
+			    int        *choice)
+{
+  gchar **messages;
+  GtkWidget *d;
+  int i, n_choices, result;
+
+  g_return_val_if_fail (message && choices && choice, FALSE);
+
+  messages = g_strsplit (message, "\n", 2);
+
+  d = gtk_message_dialog_new (parent,
+			      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+			      GTK_MESSAGE_QUESTION,
+			      GTK_BUTTONS_NONE,
+			      "%s",
+			      messages[0]);
+
+  if (g_strv_length (messages) > 1)
+    gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (d),
+						"%s",
+						messages[1]);
+  g_strfreev (messages);
+
+  n_choices = g_strv_length (choices);
+  for (i = 0; i < n_choices; i++)
+    gtk_dialog_add_button (GTK_DIALOG (d), choices[i], i);
+
+  result = gtk_dialog_run (GTK_DIALOG (d));
+  gtk_widget_destroy (d);
+
+  if (result == GTK_RESPONSE_NONE || result == GTK_RESPONSE_DELETE_EVENT)
+    return FALSE;
+
+  *choice = result;
+  return TRUE;
+}
+
+#include "vinagre-ssh.h"
+static void
+shit (void)
+{
+  vinagre_ssh_connect (NULL, -1, NULL, NULL, NULL, NULL, NULL);
+}
+
 /* vim: set ts=8: */
